@@ -4,12 +4,14 @@
 #include <termios.h>
 #include <functional>
 #include <vector>
+#include <cstdlib>
+
 
 static std::unique_ptr<GeminiFlatpanel> mydriver(new GeminiFlatpanel());
 
 GeminiFlatpanel::GeminiFlatpanel() : LightBoxInterface(this), DustCapInterface(this)
 {
-    setVersion(1, 1);
+    setVersion(1, 2);
 }
 
 const char *GeminiFlatpanel::getDefaultName()
@@ -27,6 +29,10 @@ bool GeminiFlatpanel::initProperties()
     LI::initProperties(MAIN_CONTROL_TAB, LI::CAN_DIM);
     DI::initProperties(MAIN_CONTROL_TAB);
 
+
+    // Driver interface will be set dynamically in Handshake() based on device capabilities
+    setDriverInterface(AUX_INTERFACE | LIGHTBOX_INTERFACE);
+
     // Initialize device selection property
     DeviceTypeSP.fill(
         getDeviceName(),
@@ -44,8 +50,6 @@ bool GeminiFlatpanel::initProperties()
     DeviceTypeSP[DEVICE_LITE].fill("LITE", "Lite", ISS_OFF);
     DeviceTypeSP.load();
 
-    // Driver interface will be set dynamically in Handshake() based on device capabilities
-
     addAuxControls();
 
     serialConnection = new Connection::Serial(this);
@@ -492,12 +496,11 @@ bool GeminiFlatpanel::Handshake()
         commandTerminator = adapter->getCommandTerminator();
 
         // Set driver interface based on device capabilities
-        uint32_t interface = AUX_INTERFACE | LIGHTBOX_INTERFACE;
         if (adapter && adapter->supportsDustCap())
         {
-            interface |= DUSTCAP_INTERFACE;
+            setDriverInterface(getDriverInterface() | DUSTCAP_INTERFACE);
+            syncDriverInfo();
         }
-        setDriverInterface(interface);
 
         // Get config status from adapter
         int adapterConfigStatus;
@@ -591,12 +594,11 @@ bool GeminiFlatpanel::Handshake()
     }
 
     // Set driver interface based on device capabilities
-    uint32_t interface = AUX_INTERFACE | LIGHTBOX_INTERFACE;
     if (adapter && adapter->supportsDustCap())
     {
-        interface |= DUSTCAP_INTERFACE;
+        setDriverInterface(getDriverInterface() | DUSTCAP_INTERFACE);
+        syncDriverInfo();
     }
-    setDriverInterface(interface);
 
     // Check config status using adapter
     int adapterConfigStatus;
