@@ -103,8 +103,7 @@ class RigelDome : public INDI::Dome
         ///////////////////////////////////////////////////////////////////////////////////
         /// Properties
         ///////////////////////////////////////////////////////////////////////////////////
-        ISwitchVectorProperty OperationSP;
-        ISwitch OperationS[2];
+        INDI::PropertySwitch OperationSP {2};
         enum
         {
             OPERATION_FIND_HOME,
@@ -112,8 +111,7 @@ class RigelDome : public INDI::Dome
         };
 
         // Info
-        ITextVectorProperty InfoTP;
-        IText InfoT[4] {};
+        INDI::PropertyText InfoTP {4};
         enum
         {
             INFO_FIRMWARE,
@@ -123,8 +121,7 @@ class RigelDome : public INDI::Dome
         };
 
         // Home angle
-        INumber HomePositionN[1];
-        INumberVectorProperty HomePositionNP;
+        INDI::PropertyNumber HomePositionNP {1};
 
         /////////////////////////////////////////////////////////////////////////////
         /// Static Helper Values
@@ -134,6 +131,6 @@ class RigelDome : public INDI::Dome
         static const char DRIVER_STOP_CHAR { 0x0D };
         // Wait up to a maximum of 3 seconds for serial input
         static constexpr const uint8_t DRIVER_TIMEOUT {3};
-        // Maximum buffer for sending/receving.
+        // Maximum buffer for sending/receiving.
         static constexpr const uint8_t DRIVER_LEN {64};
 };
