@@ -33,32 +33,61 @@ void ManualFilter::ISGetProperties(const char *dev)
 {
     INDI::FilterWheel::ISGetProperties(dev);
 
-    defineProperty(&MaxFiltersNP);
+    defineProperty(MaxFiltersNP);
 }
 
 bool ManualFilter::initProperties()
 {
     INDI::FilterWheel::initProperties();
 
+    // Max number of filters
+    MaxFiltersNP[0].fill("MAX", "Filters", "%.f", 1, 16, 1, 5);
+    MaxFiltersNP.fill(getDeviceName(), "MAX_FILTERS", "Max.", MAIN_CONTROL_TAB, IP_RW, 60, IPS_IDLE);
+    MaxFiltersNP.load();
+
+    FilterSlotNP[0].setMax(MaxFiltersNP[0].getValue());
+
+    // If exists filter names count is less than our desired max count. Save them then generate samples to fill the rest,
+    // then restore the current filters before we overwrote them with generateSampleFilters.
+    if (FilterNameTP.count() != MaxFiltersNP[0].getValue())
+    {
+        if (FilterNameTP.count() < MaxFiltersNP[0].getValue())
+        {
+            // Save existing labels and texts
+            std::vector<std::string> labels, values;
+            for (auto oneFilter : FilterNameTP)
+            {
+                labels.push_back(oneFilter.getLabel());
+                values.push_back(oneFilter.getText());
+            }
+
+            // Generate all samples to fill the rest
+            generateSampleFilters();
+
+            // Now restore the first filter names that we loaded from config
+            for (uint8_t i = 0; i < labels.size(); i++)
+            {
+                FilterNameTP[i].setLabel(labels[i]);
+                FilterNameTP[i].setText(values[i]);
+            }
+        }
+        else
+        {
+            FilterNameTP.resize(MaxFiltersNP[0].getValue());
+            FilterNameTP.shrink_to_fit();
+        }
+
+        // Make sure to save changes to config immediately.
+        saveConfig();
+    }
+
     // User Set Filter
-    IUFillSwitch(&FilterSetS[0], "FILTER_SET", "Filter is set", ISS_OFF);
-    IUFillSwitchVector(&FilterSetSP, FilterSetS, 1, getDeviceName(), "CONFIRM_FILTER_SET", "Confirm", MAIN_CONTROL_TAB, IP_RW,
-                       ISR_ATMOST1, 60, IPS_IDLE);
+    FilterSetSP[0].fill("FILTER_SET", "Filter is set", ISS_OFF);
+    FilterSetSP.fill(getDeviceName(), "CONFIRM_FILTER_SET", "Confirm", MAIN_CONTROL_TAB, IP_RW, ISR_ATMOST1, 60, IPS_IDLE);
 
     // Sync Filter Position. Sets current position to different position without actually changing filter.
-    IUFillNumber(&SyncN[0], "TARGET_FILTER", "Target Filter", "%.f", 1, 16, 1, 0);
-    IUFillNumberVector(&SyncNP, SyncN, 1, getDeviceName(), "SYNC_FILTER", "Sync", MAIN_CONTROL_TAB, IP_WO, 60, IPS_IDLE);
-
-    // Max number of filters
-    double maxFilters = 5;
-    IUGetConfigNumber(getDeviceName(), "MAX_FILTERS", "MAX", &maxFilters);
-    // If names are already loaded, then we ignore all and use this instead.
-    if (FilterNameTP.size() > 0)
-        maxFilters = FilterNameTP.size();
-    FilterSlotNP[0].setMax(maxFilters);
-    IUFillNumber(&MaxFiltersN[0], "MAX", "Filters", "%.f", 1, 16, 1, maxFilters);
-    IUFillNumberVector(&MaxFiltersNP, MaxFiltersN, 1, getDeviceName(), "MAX_FILTERS", "Max.", MAIN_CONTROL_TAB, IP_RW, 60,
-                       IPS_IDLE);
+    SyncNP[0].fill("TARGET_FILTER", "Target Filter", "%.f", 1, 16, 1, 0);
+    SyncNP.fill(getDeviceName(), "SYNC_FILTER", "Sync", MAIN_CONTROL_TAB, IP_WO, 60, IPS_IDLE);
 
     return true;
 }
@@ -69,17 +98,17 @@ bool ManualFilter::updateProperties()
 
     if (isConnected())
     {
-        deleteProperty(MaxFiltersNP.name);
+        deleteProperty(MaxFiltersNP);
 
-        defineProperty(&SyncNP);
-        defineProperty(&FilterSetSP);
+        defineProperty(SyncNP);
+        defineProperty(FilterSetSP);
     }
     else
     {
-        deleteProperty(SyncNP.name);
-        deleteProperty(FilterSetSP.name);
+        deleteProperty(SyncNP);
+        deleteProperty(FilterSetSP);
 
-        defineProperty(&MaxFiltersNP);
+        defineProperty(MaxFiltersNP);
     }
 
     return true;
@@ -89,26 +118,26 @@ bool ManualFilter::ISNewNumber(const char *dev, const char *name, double values[
 {
     if (!strcmp(dev, getDeviceName()))
     {
-        if (!strcmp(name, SyncNP.name))
+        if (SyncNP.isNameMatch(name))
         {
-            IUUpdateNumber(&SyncNP, values, names, n);
-            CurrentFilter = SyncN[0].value;
+            SyncNP.update(values, names, n);
+            CurrentFilter = SyncNP[0].getValue();
             FilterSlotNP[0].setValue(CurrentFilter);
             FilterSlotNP.apply();
-            SyncNP.s = IPS_OK;
-            IDSetNumber(&SyncNP, nullptr);
+            SyncNP.setState(IPS_OK);
+            SyncNP.apply();
 
             LOGF_INFO("Filter wheel is synced to slot %d", CurrentFilter);
 
             return true;
         }
-        else if (!strcmp(MaxFiltersNP.name, name))
+        else if (MaxFiltersNP.isNameMatch(name))
         {
-            IUUpdateNumber(&MaxFiltersNP, values, names, n);
-            FilterSlotNP[0].setMax(MaxFiltersN[0].value);
-            MaxFiltersNP.s = IPS_OK;
-            saveConfig(true, MaxFiltersNP.name);
-            IDSetNumber(&MaxFiltersNP, nullptr);
+            MaxFiltersNP.update(values, names, n);
+            FilterSlotNP[0].setMax(MaxFiltersNP[0].getValue());
+            MaxFiltersNP.setState(IPS_OK);
+            saveConfig(MaxFiltersNP);
+            MaxFiltersNP.apply();
 
             return true;
         }
@@ -121,11 +150,11 @@ bool ManualFilter::ISNewSwitch(const char *dev, const char *name, ISState *state
 {
     if (!strcmp(dev, getDeviceName()))
     {
-        if (!strcmp(name, FilterSetSP.name))
+        if (FilterSetSP.isNameMatch(name))
         {
             SelectFilterDone(CurrentFilter);
-            FilterSetSP.s = IPS_OK;
-            IDSetSwitch(&FilterSetSP, nullptr);
+            FilterSetSP.setState(IPS_OK);
+            FilterSetSP.apply();
             return true;
         }
 
@@ -148,8 +177,8 @@ bool ManualFilter::SelectFilter(int f)
 {
     CurrentFilter = f;
 
-    FilterSetSP.s = IPS_BUSY;
-    IDSetSwitch(&FilterSetSP, nullptr);
+    FilterSetSP.setState(IPS_BUSY);
+    FilterSetSP.apply();
     LOGF_INFO("Please change filter to %s then click Filter is set when done.", FilterNameTP[f - 1].getText());
     return true;
 }
@@ -157,8 +186,6 @@ bool ManualFilter::SelectFilter(int f)
 bool ManualFilter::saveConfigItems(FILE *fp)
 {
     INDI::FilterWheel::saveConfigItems(fp);
-
-    IUSaveConfigNumber(fp, &MaxFiltersNP);
-
+    MaxFiltersNP.save(fp);
     return true;
 }
