@@ -34,44 +34,6 @@
 
 static std::unique_ptr<MyFocuserPro2> myFocuserPro2(new MyFocuserPro2());
 
-void ISGetProperties(const char * dev)
-{
-    myFocuserPro2->ISGetProperties(dev);
-}
-
-void ISNewSwitch(const char * dev, const char * name, ISState * states, char * names[], int n)
-{
-    myFocuserPro2->ISNewSwitch(dev, name, states, names, n);
-}
-
-void ISNewText(const char * dev, const char * name, char * texts[], char * names[], int n)
-{
-    myFocuserPro2->ISNewText(dev, name, texts, names, n);
-}
-
-void ISNewNumber(const char * dev, const char * name, double values[], char * names[], int n)
-{
-    myFocuserPro2->ISNewNumber(dev, name, values, names, n);
-}
-
-void ISNewBLOB(const char * dev, const char * name, int sizes[], int blobsizes[], char * blobs[], char * formats[],
-               char * names[], int n)
-{
-    INDI_UNUSED(dev);
-    INDI_UNUSED(name);
-    INDI_UNUSED(sizes);
-    INDI_UNUSED(blobsizes);
-    INDI_UNUSED(blobs);
-    INDI_UNUSED(formats);
-    INDI_UNUSED(names);
-    INDI_UNUSED(n);
-}
-
-void ISSnoopDevice(XMLEle * root)
-{
-    myFocuserPro2->ISSnoopDevice(root);
-}
-
 MyFocuserPro2::MyFocuserPro2()
 {
     // Can move in Absolute & Relative motions, can AbortFocuser motion, and has variable speed.
@@ -81,32 +43,32 @@ MyFocuserPro2::MyFocuserPro2()
 
     setSupportedConnections(CONNECTION_SERIAL | CONNECTION_TCP);
 
-    setVersion(0, 7);
+    setVersion(CDRIVER_VERSION_MAJOR, CDRIVER_VERSION_MINOR);
 }
 
 bool MyFocuserPro2::initProperties()
 {
     INDI::Focuser::initProperties();
 
-    FocusSpeedN[0].min   = 0;
-    FocusSpeedN[0].max   = 2;
-    FocusSpeedN[0].value = 1;
+    FocusSpeedNP[0].setMin(0);
+    FocusSpeedNP[0].setMax(2);
+    FocusSpeedNP[0].setValue(1);
 
     /* Relative and absolute movement */
-    FocusRelPosN[0].min   = 0.;
-    FocusRelPosN[0].max   = 50000.;
-    FocusRelPosN[0].value = 0.;
-    FocusRelPosN[0].step  = 1000;
+    FocusRelPosNP[0].setMin(0.);
+    FocusRelPosNP[0].setMax(50000.);
+    FocusRelPosNP[0].setValue(0.);
+    FocusRelPosNP[0].setStep(1000);
 
-    FocusAbsPosN[0].min   = 0.;
-    FocusAbsPosN[0].max   = 200000.;
-    FocusAbsPosN[0].value = 0.;
-    FocusAbsPosN[0].step  = 1000;
+    FocusAbsPosNP[0].setMin(0.);
+    FocusAbsPosNP[0].setMax(200000.);
+    FocusAbsPosNP[0].setValue(0.);
+    FocusAbsPosNP[0].setStep(1000);
 
-    FocusMaxPosN[0].min   = 1024.;
-    FocusMaxPosN[0].max   = 200000.;
-    FocusMaxPosN[0].value = 0.;
-    FocusMaxPosN[0].step  = 1000;
+    FocusMaxPosNP[0].setMin(1024.);
+    FocusMaxPosNP[0].setMax(200000.);
+    FocusMaxPosNP[0].setValue(0.);
+    FocusMaxPosNP[0].setStep(1000);
 
     //Backlash
     BacklashInStepsN[0].min   = 0;
@@ -122,23 +84,23 @@ bool MyFocuserPro2::initProperties()
 
     // Backlash In
     IUFillSwitch(&BacklashInS[INDI_ENABLED], "INDI_ENABLED", "On", ISS_OFF);
-    IUFillSwitch(&BacklashInS[INDI_DISABLED], "INDI_DISABLED", "Off", ISS_OFF);
-    IUFillSwitchVector(&BacklashInSP, BacklashInS, 2, getDeviceName(), "Backlash In", "", OPTIONS_TAB, IP_RW, ISR_1OFMANY, 0,
-                       IPS_IDLE);
+    IUFillSwitch(&BacklashInS[INDI_DISABLED], "INDI_DISABLED", "Off", ISS_ON);
+    IUFillSwitchVector(&BacklashInSP, BacklashInS, 2, getDeviceName(), "BACKLASH_IN_TOGGLE", "Backlash In", SETTINGS_TAB, IP_RW,
+                       ISR_1OFMANY, 0, IPS_IDLE);
 
     IUFillNumber(&BacklashInStepsN[0], "Steps", "", "%3.0f", 0, 512, 2, 0);
-    IUFillNumberVector(&BacklashInStepsNP, BacklashInStepsN, 1, getDeviceName(), "Backlash-In", "", OPTIONS_TAB, IP_RW, 0,
-                       IPS_IDLE);
+    IUFillNumberVector(&BacklashInStepsNP, BacklashInStepsN, 1, getDeviceName(), "BACKLASH_IN_VALUE", "Backlash In",
+                       SETTINGS_TAB, IP_RW, 0,  IPS_IDLE);
 
     // Backlash Out
     IUFillSwitch(&BacklashOutS[INDI_ENABLED], "INDI_ENABLED", "On", ISS_OFF);
-    IUFillSwitch(&BacklashOutS[INDI_DISABLED], "INDI_DISABLED", "Off", ISS_OFF);
-    IUFillSwitchVector(&BacklashOutSP, BacklashOutS, 2, getDeviceName(), "Backlash Out", "", OPTIONS_TAB, IP_RW, ISR_1OFMANY, 0,
-                       IPS_IDLE);
+    IUFillSwitch(&BacklashOutS[INDI_DISABLED], "INDI_DISABLED", "Off", ISS_ON);
+    IUFillSwitchVector(&BacklashOutSP, BacklashOutS, 2, getDeviceName(), "BACKLASH_OUT_TOGGLE", "Backlash Out", SETTINGS_TAB,
+                       IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
 
     IUFillNumber(&BacklashOutStepsN[0], "Steps", "", "%3.0f", 0, 512, 2, 0);
-    IUFillNumberVector(&BacklashOutStepsNP, BacklashOutStepsN, 1, getDeviceName(), "Backlash-Out", "", OPTIONS_TAB, IP_RW, 0,
-                       IPS_IDLE);
+    IUFillNumberVector(&BacklashOutStepsNP, BacklashOutStepsN, 1, getDeviceName(), "BACKLASH_OUT_VALUE", "Backlash Out",
+                       SETTINGS_TAB, IP_RW, 0, IPS_IDLE);
 
     // Focuser temperature
     IUFillNumber(&TemperatureN[0], "TEMPERATURE", "Celsius", "%6.2f", -40, 80., 0., 0.);
@@ -147,38 +109,42 @@ bool MyFocuserPro2::initProperties()
 
     // Temperature Settings
     IUFillNumber(&TemperatureSettingN[0], "Coefficient", "", "%6.2f", 0, 50, 1, 0);
-    IUFillNumberVector(&TemperatureSettingNP, TemperatureSettingN, 1, getDeviceName(), "T. Settings", "", OPTIONS_TAB, IP_RW, 0,
-                       IPS_IDLE);
+    IUFillNumberVector(&TemperatureSettingNP, TemperatureSettingN, 1, getDeviceName(), "FOCUS_TEMPERATURE_SETTINGS",
+                       "T. Settings", SETTINGS_TAB, IP_RW, 0, IPS_IDLE);
 
     // Compensate for temperature
     IUFillSwitch(&TemperatureCompensateS[TEMP_COMPENSATE_ENABLE], "TEMP_COMPENSATE_ENABLE", "Enable", ISS_OFF);
-    IUFillSwitch(&TemperatureCompensateS[TEMP_COMPENSATE_DISABLE], "TEMP_COMPENSATE_DISABLE", "Disable", ISS_OFF);
-    IUFillSwitchVector(&TemperatureCompensateSP, TemperatureCompensateS, 2, getDeviceName(), "T. Compensate", "", OPTIONS_TAB,
-                       IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
-
-    // Step Mode
-    IUFillSwitch(&StepModeS[FOCUS_THIRTYSECOND_STEP], "FOCUS_THIRTYSECOND_STEP", "1/32 Step", ISS_OFF);
-    IUFillSwitch(&StepModeS[FOCUS_SIXTEENTH_STEP], "FOCUS_SIXTEENTH_STEP", "1/16 Step", ISS_OFF);
-    IUFillSwitch(&StepModeS[FOCUS_EIGHTH_STEP], "FOCUS_EIGHTH_STEP", "1/8 Step", ISS_OFF);
-    IUFillSwitch(&StepModeS[FOCUS_QUARTER_STEP], "FOCUS_QUARTER_STEP", "1/4 Step", ISS_OFF);
-    IUFillSwitch(&StepModeS[FOCUS_HALF_STEP], "FOCUS_HALF_STEP", "1/2 Step", ISS_OFF);
-    IUFillSwitch(&StepModeS[FOCUS_FULL_STEP], "FOCUS_FULL_STEP", "Full Step", ISS_OFF);
-    IUFillSwitchVector(&StepModeSP, StepModeS, 6, getDeviceName(), "Step Mode", "", OPTIONS_TAB, IP_RW, ISR_1OFMANY, 0,
-                       IPS_IDLE);
+    IUFillSwitch(&TemperatureCompensateS[TEMP_COMPENSATE_DISABLE], "TEMP_COMPENSATE_DISABLE", "Disable", ISS_ON);
+    IUFillSwitchVector(&TemperatureCompensateSP, TemperatureCompensateS, 2, getDeviceName(), "FOCUS_TEMPERATURE_COMPENSATION",
+                       "T. Compensation", SETTINGS_TAB,  IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
 
+    // Add step modes 1/64 to 1/256 for TMC type drivers R Brown June 2021
+    IUFillSwitch(&StepModeS[TWOHUNDREDFIFTYSIX_STEP], "TWOHUNDREDFIFTYSIX_STEP", "1/256 Step", ISS_OFF);
+    IUFillSwitch(&StepModeS[ONEHUNDREDTWENTYEIGHT_STEP], "ONEHUNDREDTWENTYEIGHT_STEP", "1/128 Step", ISS_OFF);
+    IUFillSwitch(&StepModeS[SIXTYFOUR_STEP], "SIXTYFOUR_STEP", "1/64 Step", ISS_OFF);
+    IUFillSwitch(&StepModeS[THIRTYSECOND_STEP], "THIRTYSECOND_STEP", "1/32 Step", ISS_OFF);
+    IUFillSwitch(&StepModeS[SIXTEENTH_STEP], "SIXTEENTH_STEP", "1/16 Step", ISS_OFF);
+    IUFillSwitch(&StepModeS[EIGHTH_STEP], "EIGHTH_STEP", "1/8 Step", ISS_OFF);
+    IUFillSwitch(&StepModeS[QUARTER_STEP], "QUARTER_STEP", "1/4 Step", ISS_OFF);
+    IUFillSwitch(&StepModeS[HALF_STEP], "HALF_STEP", "1/2 Step", ISS_OFF);
+    IUFillSwitch(&StepModeS[FULL_STEP], "FULL_STEP", "Full Step", ISS_OFF);
+    IUFillSwitchVector(&StepModeSP, StepModeS, 9, getDeviceName(), "FOCUS_STEP_MODE", "Step Mode", SETTINGS_TAB, IP_RW,
+                       ISR_1OFMANY, 0, IPS_IDLE);
 
     IUFillSwitch(&CoilPowerS[COIL_POWER_ON], "COIL_POWER_ON", "On", ISS_OFF);
-    IUFillSwitch(&CoilPowerS[COIL_POWER_OFF], "COIL_POWER_OFF", "Off", ISS_OFF);
-    IUFillSwitchVector(&CoilPowerSP, CoilPowerS, 2, getDeviceName(), "Coil Power", "", OPTIONS_TAB, IP_RW, ISR_1OFMANY, 0,
+    IUFillSwitch(&CoilPowerS[COIL_POWER_OFF], "COIL_POWER_OFF", "Off", ISS_ON);
+    IUFillSwitchVector(&CoilPowerSP, CoilPowerS, 2, getDeviceName(), "FOCUS_COIL_POWER", "Coil Power", SETTINGS_TAB, IP_RW,
+                       ISR_1OFMANY, 0,
                        IPS_IDLE);
 
     IUFillSwitch(&DisplayS[DISPLAY_OFF], "DISPLAY_OFF", "Off", ISS_OFF);
-    IUFillSwitch(&DisplayS[DISPLAY_ON], "DISPLAY_ON", "On", ISS_OFF);
-    IUFillSwitchVector(&DisplaySP, DisplayS, 2, getDeviceName(), "Display", "", OPTIONS_TAB, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
+    IUFillSwitch(&DisplayS[DISPLAY_ON], "DISPLAY_ON", "On", ISS_ON);
+    IUFillSwitchVector(&DisplaySP, DisplayS, 2, getDeviceName(), "FOCUS_DISPLAY", "Display", SETTINGS_TAB, IP_RW, ISR_1OFMANY,
+                       0, IPS_IDLE);
 
 
     IUFillSwitch(&GotoHomeS[0], "GOTO_HOME", "Go", ISS_OFF);
-    IUFillSwitchVector(&GotoHomeSP, GotoHomeS, 1, getDeviceName(), "Goto Home Position", "", MAIN_CONTROL_TAB, IP_RW,
+    IUFillSwitchVector(&GotoHomeSP, GotoHomeS, 1, getDeviceName(), "FOCUS_HOME", "Home", MAIN_CONTROL_TAB, IP_RW,
                        ISR_ATMOST1, 0, IPS_IDLE);
 
     setPollingPeriodRange(1000, 30000);
@@ -196,17 +162,17 @@ bool MyFocuserPro2::updateProperties()
 
     if (isConnected())
     {
-        defineSwitch(&GotoHomeSP);
-        defineNumber(&TemperatureNP);
-        defineNumber(&TemperatureSettingNP);
-        defineSwitch(&TemperatureCompensateSP);
-        defineSwitch(&BacklashInSP);
-        defineNumber(&BacklashInStepsNP);
-        defineSwitch(&BacklashOutSP);
-        defineNumber(&BacklashOutStepsNP);
-        defineSwitch(&StepModeSP);
-        defineSwitch(&DisplaySP);
-        defineSwitch(&CoilPowerSP);
+        defineProperty(&GotoHomeSP);
+        defineProperty(&TemperatureNP);
+        defineProperty(&TemperatureSettingNP);
+        defineProperty(&TemperatureCompensateSP);
+        defineProperty(&BacklashInSP);
+        defineProperty(&BacklashInStepsNP);
+        defineProperty(&BacklashOutSP);
+        defineProperty(&BacklashOutStepsNP);
+        defineProperty(&StepModeSP);
+        defineProperty(&DisplaySP);
+        defineProperty(&CoilPowerSP);
 
         setTemperatureCelsius();
 
@@ -277,16 +243,25 @@ bool MyFocuserPro2::Ack()
         if(!transmissionSuccess)
         {
             tty_error_msg(rc, errstr, MAXRBUF);
-            LOGF_ERROR("Handshake Attempt %i, tty transmission error: %s.", numChecks, errstr);
+            LOGF_ERROR("Handshake Attempt %i, Connection transmission error: %s.", numChecks, errstr);
+        }
+
+        bool responseSuccess;
+        if (getActiveConnection()->type() == Connection::Interface::CONNECTION_SERIAL)
+        {
+            responseSuccess = (rc = tty_read(PortFD, resp, 5, MYFOCUSERPRO2_SERIAL_TIMEOUT, &nbytes_read)) == TTY_OK;
+        }
+        else
+        {
+            // assume TCPIP Connection
+            responseSuccess = (rc = tty_read(PortFD, resp, 5, MYFOCUSERPRO2_TCPIP_TIMEOUT, &nbytes_read)) == TTY_OK;
         }
 
-        bool responseSuccess = (rc = tty_read(PortFD, resp, 5, ML_TIMEOUT, &nbytes_read)) == TTY_OK;
         if(!responseSuccess)
         {
             tty_error_msg(rc, errstr, MAXRBUF);
             LOGF_ERROR("Handshake Attempt %i, updatePosition response error: %s.", numChecks, errstr);
         }
-
         success = transmissionSuccess && responseSuccess;
     }
 
@@ -302,26 +277,10 @@ bool MyFocuserPro2::Ack()
 
     if (rc > 0)
     {
-        // Minimum check only applicable to serial version
-        if (getActiveConnection()->type() == Connection::Interface::CONNECTION_SERIAL)
-        {
-            if(firmWareVersion >= MINIMUM_FIRMWARE_VERSION)
-            {
-                LOGF_INFO("MyFP2 reported firmware %d", firmWareVersion);
-                return true;
-
-            }
-            else
-            {
-                LOGF_ERROR("Invalid Firmware: focuser firmware version value %d, minimum supported is %d", firmWareVersion,
-                           MINIMUM_FIRMWARE_VERSION );
-            }
-        }
-        else
-        {
-            LOG_INFO("Connection to network focuser is successful.");
-            return true;
-        }
+        // remove check for firmware => 291, assume user is not using older firmware
+        LOGF_INFO("MyFP2 reported firmware %d", firmWareVersion);
+        LOG_INFO("Connection to focuser is successful.");
+        return true;
     }
     else
     {
@@ -335,30 +294,35 @@ bool MyFocuserPro2::readCoilPowerState()
     char res[ML_RES] = {0};
 
     if (sendCommand(":11#", res) == false)
+    {
         return false;
+    }
 
     uint32_t temp = 0;
 
     int rc = sscanf(res, "O%u#", &temp);
 
     if (rc > 0)
-
+    {
         if(temp == 0)
+        {
             CoilPowerS[COIL_POWER_OFF].s = ISS_ON;
+        }
         else if (temp == 1)
+        {
             CoilPowerS[COIL_POWER_ON].s = ISS_ON;
+        }
         else
         {
             LOGF_ERROR("Invalid Response: focuser Coil Power value (%s)", res);
             return false;
         }
+    }
     else
     {
         LOGF_ERROR("Unknown error: focuser Coil Power value (%s)", res);
         return false;
     }
-
-
     return true;
 }
 
@@ -374,26 +338,26 @@ bool MyFocuserPro2::readReverseDirection()
     int rc = sscanf(res, "R%u#", &temp);
 
     if (rc > 0)
-
+    {
         if(temp == 0)
         {
-            FocusReverseS[INDI_DISABLED].s = ISS_ON;
+            FocusReverseSP[INDI_DISABLED].setState(ISS_ON);
         }
         else if (temp == 1)
         {
-            FocusReverseS[INDI_ENABLED].s = ISS_ON;
+            FocusReverseSP[INDI_ENABLED].setState(ISS_ON);
         }
         else
         {
             LOGF_ERROR("Invalid Response: focuser Reverse direction value (%s)", res);
             return false;
         }
+    }
     else
     {
         LOGF_ERROR("Unknown error: focuser Reverse direction value (%s)", res);
         return false;
     }
-
     return true;
 }
 
@@ -402,26 +366,56 @@ bool MyFocuserPro2::readStepMode()
     char res[ML_RES] = {0};
 
     if (sendCommand(":29#", res) == false)
-        return false;
-
-    if (strcmp(res, "S1#") == 0)
-        StepModeS[FOCUS_FULL_STEP].s = ISS_ON;
-    else if (strcmp(res, "S2#") == 0)
-        StepModeS[FOCUS_HALF_STEP].s = ISS_ON;
-    else if (strcmp(res, "S4#") == 0)
-        StepModeS[FOCUS_QUARTER_STEP].s = ISS_ON;
-    else if (strcmp(res, "S8#") == 0)
-        StepModeS[FOCUS_EIGHTH_STEP].s = ISS_ON;
-    else if (strcmp(res, "S16#") == 0)
-        StepModeS[FOCUS_SIXTEENTH_STEP].s = ISS_ON;
-    else if (strcmp(res, "S32#") == 0)
-        StepModeS[FOCUS_THIRTYSECOND_STEP].s = ISS_ON;
-    else
     {
-        LOGF_ERROR("Unknown error: focuser Step Mode value (%s)", res);
         return false;
     }
 
+    uint32_t stepmode = 0;
+
+    int rc = sscanf(res, "S%u#", &stepmode);
+
+    if( rc > 0 )
+    {
+        switch( stepmode )
+        {
+            case STEPMODE_FULL:
+                StepModeS[FULL_STEP].s = ISS_ON;
+                break;
+            case STEPMODE_HALF:
+                StepModeS[HALF_STEP].s = ISS_ON;
+                break;
+            case STEPMODE_QUARTER:
+                StepModeS[QUARTER_STEP].s = ISS_ON;
+                break;
+            case STEPMODE_EIGHTH:
+                StepModeS[EIGHTH_STEP].s = ISS_ON;
+                break;
+            case STEPMODE_SIXTEENTH:
+                StepModeS[SIXTEENTH_STEP].s = ISS_ON;
+                break;
+            case STEPMODE_THIRTYSECOND:
+                StepModeS[THIRTYSECOND_STEP].s = ISS_ON;
+                break;
+            case STEPMODE_SIXTYFOUR:
+                StepModeS[SIXTYFOUR_STEP].s = ISS_ON;
+                break;
+            case STEPMODE_ONEHUNDREDTWENTYEIGHT:
+                StepModeS[ONEHUNDREDTWENTYEIGHT_STEP].s = ISS_ON;
+                break;
+            case STEPMODE_TWOHUNDREDFIFTYSIX:
+                StepModeS[TWOHUNDREDFIFTYSIX_STEP].s = ISS_ON;
+                break;
+            default:
+                LOGF_ERROR("Unknown error: Step mode value (%d)", stepmode);
+                return false;
+                break;
+        }
+    }
+    else
+    {
+        LOGF_ERROR("Unknown error: Step mode value (%s)", res);
+        return false;
+    }
     return true;
 }
 
@@ -435,14 +429,14 @@ bool MyFocuserPro2::readTemperature()
     double temp = 0;
     int rc = sscanf(res, "Z%lf#", &temp);
     if (rc > 0)
-        // Signed hex
+    {
         TemperatureN[0].value = temp;
+    }
     else
     {
         LOGF_ERROR("Unknown error: focuser temperature value (%s)", res);
         return false;
     }
-
     return true;
 }
 
@@ -451,29 +445,35 @@ bool MyFocuserPro2::readTempCompensateEnable()
     char res[ML_RES] = {0};
 
     if (sendCommand(":24#", res) == false)
+    {
         return false;
+    }
 
     uint32_t temp = 0;
 
     int rc = sscanf(res, "1%u#", &temp);
 
     if (rc > 0)
-
+    {
         if(temp == 0)
+        {
             TemperatureCompensateS[TEMP_COMPENSATE_DISABLE].s = ISS_ON;
+        }
         else if (temp == 1)
+        {
             TemperatureCompensateS[TEMP_COMPENSATE_ENABLE].s = ISS_ON;
+        }
         else
         {
             LOGF_ERROR("Invalid Response: focuser T.Compensate value (%s)", res);
             return false;
         }
+    }
     else
     {
         LOGF_ERROR("Unknown error: focuser T.Compensate value (%s)", res);
         return false;
     }
-
     return true;
 }
 
@@ -483,40 +483,46 @@ bool MyFocuserPro2::readPosition()
     char res[ML_RES] = {0};
 
     if (sendCommand(":00#", res) == false)
+    {
         return false;
+    }
 
     int32_t pos;
     int rc = sscanf(res, "%*c%d#", &pos);
 
     if (rc > 0)
-        FocusAbsPosN[0].value = pos;
+    {
+        FocusAbsPosNP[0].setValue(pos);
+    }
     else
     {
         LOGF_ERROR("Unknown error: focuser position value (%s)", res);
         return false;
     }
-
     return true;
 }
 
-bool MyFocuserPro2::readTempeartureCoefficient()
+bool MyFocuserPro2::readTemperatureCoefficient()
 {
     char res[ML_RES] = {0};
 
     if (sendCommand(":26#", res) == false)
+    {
         return false;
+    }
 
     int32_t val;
     int rc = sscanf(res, "B%d#", &val);
 
     if (rc > 0)
+    {
         TemperatureSettingN[0].value = val;
+    }
     else
     {
         LOGF_ERROR("Unknown error: Temperature Coefficient value (%s)", res);
         return false;
     }
-
     return true;
 }
 
@@ -525,21 +531,22 @@ bool MyFocuserPro2::readSpeed()
     char res[ML_RES] = {0};
 
     if (sendCommand(":43#", res) == false)
+    {
         return false;
+    }
 
     int speed = 0;
     int rc = sscanf(res, "C%d#", &speed);
 
     if (rc > 0)
     {
-        FocusSpeedN[0].value = speed;
+        FocusSpeedNP[0].setValue(speed);
     }
     else
     {
         LOGF_ERROR("Unknown error: focuser speed value (%s)", res);
         return false;
     }
-
     return true;
 }
 
@@ -548,14 +555,16 @@ bool MyFocuserPro2::readMaxPos()
     char res[ML_RES] = {0};
 
     if (sendCommand(":08#", res) == false)
+    {
         return false;
+    }
 
     uint32_t maxPos = 0;
     int rc = sscanf(res, "M%u#", &maxPos);
 
     if (rc > 0)
     {
-        FocusMaxPosN[0].value = maxPos;
+        FocusMaxPosNP[0].setValue(maxPos);
         Focuser::SyncPresets(maxPos);
     }
     else
@@ -563,7 +572,6 @@ bool MyFocuserPro2::readMaxPos()
         LOGF_ERROR("Unknown error: focuser max position value (%s)", res);
         return false;
     }
-
     return true;
 }
 
@@ -572,7 +580,9 @@ bool MyFocuserPro2::readBacklashInSteps()
     char res[ML_RES] = {0};
 
     if (sendCommand(":78#", res) == false)
+    {
         return false;
+    }
 
     uint32_t backlash = 0;
     int rc = sscanf(res, "6%u#", &backlash);
@@ -587,7 +597,6 @@ bool MyFocuserPro2::readBacklashInSteps()
         LOGF_ERROR("Unknown error: focuser Backlash IN value (%s)", res);
         return false;
     }
-
     return true;
 }
 
@@ -596,7 +605,9 @@ bool MyFocuserPro2::readBacklashInEnabled()
     char res[ML_RES] = {0};
 
     if (sendCommand(":74#", res) == false)
+    {
         return false;
+    }
 
     uint32_t temp = 0;
     int rc = sscanf(res, "4%u#", &temp);
@@ -604,11 +615,17 @@ bool MyFocuserPro2::readBacklashInEnabled()
     if (rc > 0)
     {
         if(temp == 0)
+        {
             BacklashInS[INDI_DISABLED].s = ISS_ON;
+        }
         else if (temp == 1)
+        {
             BacklashInS[INDI_ENABLED].s = ISS_ON;
+        }
         else
-            LOGF_ERROR("Unknown Repsonse: focuser Backlash IN enabled (%s)", res);
+        {
+            LOGF_ERROR("Unknown Response: focuser Backlash IN enabled (%s)", res);
+        }
         return false;
     }
     else
@@ -616,6 +633,7 @@ bool MyFocuserPro2::readBacklashInEnabled()
         LOGF_ERROR("Unknown error: focuser Backlash IN enabled (%s)", res);
         return false;
     }
+    return true;                // fix missing return statement R Brown June 2020
 }
 
 bool MyFocuserPro2::readBacklashOutSteps()
@@ -623,7 +641,9 @@ bool MyFocuserPro2::readBacklashOutSteps()
     char res[ML_RES] = {0};
 
     if (sendCommand(":80#", res) == false)
+    {
         return false;
+    }
 
     uint32_t backlash = 0;
     int rc = sscanf(res, "7%u#", &backlash);
@@ -637,7 +657,6 @@ bool MyFocuserPro2::readBacklashOutSteps()
         LOGF_ERROR("Unknown error: focuser Backlash OUT value (%s)", res);
         return false;
     }
-
     return true;
 }
 
@@ -646,7 +665,9 @@ bool MyFocuserPro2::readBacklashOutEnabled()
     char res[ML_RES] = {0};
 
     if (sendCommand(":76#", res) == false)
+    {
         return false;
+    }
 
     uint32_t temp = 0;
     int rc = sscanf(res, "5%u#", &temp);
@@ -654,11 +675,17 @@ bool MyFocuserPro2::readBacklashOutEnabled()
     if (rc > 0)
     {
         if(temp == 0)
+        {
             BacklashOutS[INDI_DISABLED].s = ISS_ON;
+        }
         else if (temp == 1)
+        {
             BacklashOutS[INDI_ENABLED].s = ISS_ON;
+        }
         else
+        {
             LOGF_ERROR("Unknown response: focuser Backlash OUT enabled (%s)", res);
+        }
         return false;
     }
     else
@@ -666,7 +693,7 @@ bool MyFocuserPro2::readBacklashOutEnabled()
         LOGF_ERROR("Unknown error: focuser Backlash OUT enabled (%s)", res);
         return false;
     }
-
+    return true;                // fix missing return statement R Brown June 2020
 }
 
 bool MyFocuserPro2::readDisplayVisible()
@@ -674,7 +701,9 @@ bool MyFocuserPro2::readDisplayVisible()
     char res[ML_RES] = {0};
 
     if (sendCommand(":37#", res) == false)
+    {
         return false;
+    }
 
     uint32_t temp = 0;
 
@@ -683,9 +712,13 @@ bool MyFocuserPro2::readDisplayVisible()
     if (rc > 0)
     {
         if(temp == 0)
+        {
             DisplayS[DISPLAY_OFF].s = ISS_ON;
+        }
         else if (temp == 1)
+        {
             DisplayS[DISPLAY_ON].s = ISS_ON;
+        }
         else
         {
             LOGF_ERROR("Invalid Response: focuser Display value (%s)", res);
@@ -697,7 +730,6 @@ bool MyFocuserPro2::readDisplayVisible()
         LOGF_ERROR("Unknown error: focuser Display value (%s)", res);
         return false;
     }
-
     return true;
 }
 
@@ -705,8 +737,12 @@ bool MyFocuserPro2::isMoving()
 {
     char res[ML_RES] = {0};
 
+    readPosition(); // Fix for Ekos autofocus
+
     if (sendCommand(":01#", res) == false)
+    {
         return false;
+    }
 
     uint32_t temp = 0;
 
@@ -715,9 +751,13 @@ bool MyFocuserPro2::isMoving()
     if (rc > 0)
     {
         if(temp == 0)
+        {
             return false;
+        }
         else if (temp == 1)
+        {
             return true;
+        }
         else
         {
             LOGF_ERROR("Invalid Response: focuser isMoving value (%s)", res);
@@ -731,11 +771,10 @@ bool MyFocuserPro2::isMoving()
     }
 }
 
-
 bool MyFocuserPro2::setTemperatureCelsius()
 {
     char cmd[ML_RES] = {0};
-    snprintf(cmd, ML_RES, ":161#");
+    snprintf(cmd, ML_RES, ":16#");          // fix R Brown July 2021
     return sendCommand(cmd);
 }
 
@@ -796,19 +835,18 @@ bool MyFocuserPro2::setBacklashOutEnabled(bool enabled)
 bool MyFocuserPro2::setCoilPowerState(CoilPower enable)
 {
     char cmd[ML_RES] = {0};
-    snprintf(cmd, ML_RES, ":12%01d#", static_cast<int>(enable));
+    //snprintf(cmd, ML_RES, ":12%d#", static_cast<int>(enable));
+    snprintf(cmd, ML_RES, ":12%d#", enable);
     return sendCommand(cmd);
 }
 
-
 bool MyFocuserPro2::ReverseFocuser(bool enable)
 {
     char cmd[ML_RES] = {0};
-    snprintf(cmd, ML_RES, ":14%c#", enable ? '1' : '0');
+    snprintf(cmd, ML_RES, ":14%d#", static_cast<int>(enable));
     return sendCommand(cmd);
 }
 
-
 bool MyFocuserPro2::setDisplayVisible(DisplayMode enable)
 {
     char cmd[ML_RES] = {0};
@@ -827,11 +865,47 @@ bool MyFocuserPro2::setGotoHome()
     return sendCommand(cmd);
 }
 
-bool MyFocuserPro2::setStepMode(FocusStepMode mode)
+bool MyFocuserPro2::setStepMode(FocusStepMode smode)
 {
     char cmd[ML_RES] = {0};
-    int setMode = 1 << static_cast<int>(mode);
-    snprintf(cmd, ML_RES, ":30%02d#", setMode);
+    //int setMode = 1 << static_cast<int>(mode);
+    //snprintf(cmd, ML_RES, ":30%d#", setMode);           // fix for step modes 64-256 R Brown July 2021
+    int stepmode = 1;
+    switch( smode )
+    {
+        case FULL_STEP:
+            stepmode = 1;
+            break;
+        case HALF_STEP:
+            stepmode = 2;
+            break;
+        case QUARTER_STEP:
+            stepmode = 4;
+            break;
+        case EIGHTH_STEP:
+            stepmode = 8;
+            break;
+        case SIXTEENTH_STEP:
+            stepmode = 16;
+            break;
+        case THIRTYSECOND_STEP:
+            stepmode = 32;
+            break;
+        case SIXTYFOUR_STEP:
+            stepmode = 64;
+            break;
+        case ONEHUNDREDTWENTYEIGHT_STEP:
+            stepmode = 128;
+            break;
+        case TWOHUNDREDFIFTYSIX_STEP:
+            stepmode = 256;
+            break;
+        default:
+            LOGF_ERROR("Invalid step mode: (%d)", smode);
+            return false;
+            break;
+    }
+    snprintf(cmd, ML_RES, ":30%d#", stepmode);
     return sendCommand(cmd);
 }
 
@@ -930,7 +1004,6 @@ bool MyFocuserPro2::ISNewSwitch(const char * dev, const char * name, ISState * s
             return true;
         }
 
-
         // Display Control
         if (strcmp(DisplaySP.name, name) == 0)
         {
@@ -1109,7 +1182,7 @@ void MyFocuserPro2::getStartupValues()
     readMaxPos();
     readPosition();
     readTemperature();
-    readTempeartureCoefficient();
+    readTemperatureCoefficient();
     readSpeed();
     readTempCompensateEnable();
     readStepMode();
@@ -1137,7 +1210,6 @@ bool MyFocuserPro2::SetFocuserMaxPosition(uint32_t maxPos)
     if(sendCommand(cmd))
     {
         Focuser::SyncPresets(maxPos);
-
         return true;
     }
     return false;
@@ -1145,18 +1217,24 @@ bool MyFocuserPro2::SetFocuserMaxPosition(uint32_t maxPos)
 
 IPState MyFocuserPro2::MoveFocuser(FocusDirection dir, int speed, uint16_t duration)
 {
-    if (speed != static_cast<int>(FocusSpeedN[0].value))
+    if (speed != static_cast<int>(FocusSpeedNP[0].getValue()))
     {
         if (!setSpeed(speed))
+        {
             return IPS_ALERT;
+        }
     }
 
     // either go all the way in or all the way out
     // then use timer to stop
     if (dir == FOCUS_INWARD)
+    {
         MoveFocuser(0);
+    }
     else
-        MoveFocuser(FocusMaxPosN[0].value);
+    {
+        MoveFocuser(FocusMaxPosNP[0].getValue());
+    }
 
     IEAddTimer(duration, &MyFocuserPro2::timedMoveHelper, this);
     return IPS_BUSY;
@@ -1170,13 +1248,13 @@ void MyFocuserPro2::timedMoveHelper(void * context)
 void MyFocuserPro2::timedMoveCallback()
 {
     AbortFocuser();
-    FocusAbsPosNP.s = IPS_IDLE;
-    FocusRelPosNP.s = IPS_IDLE;
-    FocusTimerNP.s = IPS_IDLE;
-    FocusTimerN[0].value = 0;
-    IDSetNumber(&FocusAbsPosNP, nullptr);
-    IDSetNumber(&FocusRelPosNP, nullptr);
-    IDSetNumber(&FocusTimerNP, nullptr);
+    FocusAbsPosNP.setState(IPS_IDLE);
+    FocusRelPosNP.setState(IPS_IDLE);
+    FocusTimerNP.setState(IPS_IDLE);
+    FocusTimerNP[0].setValue(0);
+    FocusAbsPosNP.apply();
+    FocusRelPosNP.apply();
+    FocusTimerNP.apply();
 }
 
 IPState MyFocuserPro2::MoveAbsFocuser(uint32_t targetTicks)
@@ -1184,8 +1262,9 @@ IPState MyFocuserPro2::MoveAbsFocuser(uint32_t targetTicks)
     targetPos = targetTicks;
 
     if (!MoveFocuser(targetPos))
+    {
         return IPS_ALERT;
-
+    }
     return IPS_BUSY;
 }
 
@@ -1194,63 +1273,93 @@ IPState MyFocuserPro2::MoveRelFocuser(FocusDirection dir, uint32_t ticks)
     int32_t newPosition = 0;
 
     if (dir == FOCUS_INWARD)
-        newPosition = FocusAbsPosN[0].value - ticks;
+    {
+        newPosition = FocusAbsPosNP[0].getValue() - ticks;
+    }
     else
-        newPosition = FocusAbsPosN[0].value + ticks;
+    {
+        newPosition = FocusAbsPosNP[0].getValue() + ticks;
+    }
 
     // Clamp
-    newPosition = std::max(0, std::min(static_cast<int32_t>(FocusAbsPosN[0].max), newPosition));
+    newPosition = std::max(0, std::min(static_cast<int32_t>(FocusAbsPosNP[0].getMax()), newPosition));
     if (!MoveFocuser(newPosition))
+    {
         return IPS_ALERT;
+    }
 
-    FocusRelPosN[0].value = ticks;
-    FocusRelPosNP.s       = IPS_BUSY;
+    FocusRelPosNP[0].setValue(ticks);
+    FocusRelPosNP.setState(IPS_BUSY);
 
     return IPS_BUSY;
 }
 
+// this is occurring every 500ms
 void MyFocuserPro2::TimerHit()
 {
     if (!isConnected())
     {
-        SetTimer(POLLMS);
+        SetTimer(getCurrentPollingPeriod());
         return;
     }
 
-    bool rc = readPosition();
-    if (rc)
+    // update position every 1s
+    if (Position_Counter++ == GET_POSITION_FREQ)
     {
-        if (fabs(lastPos - FocusAbsPosN[0].value) > 5)
+        //LOG_INFO("Focuser refresh position.");
+        Position_Counter = 0;
+        bool rc = readPosition();
+        if (rc)
         {
-            IDSetNumber(&FocusAbsPosNP, nullptr);
-            lastPos = FocusAbsPosN[0].value;
+            if (fabs(lastPos - FocusAbsPosNP[0].getValue()) > 5)
+            {
+                FocusAbsPosNP.apply();
+                lastPos = FocusAbsPosNP[0].getValue();
+            }
+        }
+        if (FocusAbsPosNP.getState() == IPS_BUSY || FocusRelPosNP.getState() == IPS_BUSY)
+        {
+            if (!isMoving())
+            {
+                FocusAbsPosNP.setState(IPS_OK);
+                FocusRelPosNP.setState(IPS_OK);
+                FocusAbsPosNP.apply();
+                FocusRelPosNP.apply();
+                lastPos = FocusAbsPosNP[0].getValue();
+                LOG_INFO("Focuser reached requested position.");
+            }
         }
     }
 
-    rc = readTemperature();
-    if (rc)
-    {
-        if (fabs(lastTemperature - TemperatureN[0].value) >= 0.5)
-        {
-            IDSetNumber(&TemperatureNP, nullptr);
-            lastTemperature = TemperatureN[0].value;
-        }
-    }
-
-    if (FocusAbsPosNP.s == IPS_BUSY || FocusRelPosNP.s == IPS_BUSY)
+    if (FocusAbsPosNP.getState() == IPS_BUSY || FocusRelPosNP.getState() == IPS_BUSY)
     {
         if (!isMoving())
         {
-            FocusAbsPosNP.s = IPS_OK;
-            FocusRelPosNP.s = IPS_OK;
-            IDSetNumber(&FocusAbsPosNP, nullptr);
-            IDSetNumber(&FocusRelPosNP, nullptr);
-            lastPos = FocusAbsPosN[0].value;
+            FocusAbsPosNP.setState(IPS_OK);
+            FocusRelPosNP.setState(IPS_OK);
+            FocusAbsPosNP.apply();
+            FocusRelPosNP.apply();
+            lastPos = FocusAbsPosNP[0].getValue();
             LOG_INFO("Focuser reached requested position.");
         }
     }
 
-    SetTimer(POLLMS);
+    // update temperature every 5s
+    if (Temperature_Counter++ == GET_TEMPERATURE_FREQ)
+    {
+        //LOG_INFO("Focuser refresh Temperature.");
+        Temperature_Counter = 0;
+        bool rc = readTemperature();
+        if (rc)
+        {
+            if (fabs(lastTemperature - TemperatureN[0].value) >= 0.5)
+            {
+                IDSetNumber(&TemperatureNP, nullptr);
+                lastTemperature = TemperatureN[0].value;
+            }
+        }
+    }
+    SetTimer(getCurrentPollingPeriod());
 }
 
 bool MyFocuserPro2::AbortFocuser()
@@ -1262,11 +1371,72 @@ bool MyFocuserPro2::saveConfigItems(FILE * fp)
 {
     Focuser::saveConfigItems(fp);
 
+    IUSaveConfigNumber(fp, &TemperatureSettingNP);
+    IUSaveConfigSwitch(fp, &TemperatureCompensateSP);
+    IUSaveConfigSwitch(fp, &BacklashInSP);
+    IUSaveConfigNumber(fp, &BacklashInStepsNP);
+    IUSaveConfigSwitch(fp, &BacklashOutSP);
+    IUSaveConfigNumber(fp, &BacklashOutStepsNP);
     IUSaveConfigSwitch(fp, &StepModeSP);
+    IUSaveConfigSwitch(fp, &DisplaySP);
 
     return true;
 }
 
+// sleep for a number of milliseconds
+int MyFocuserPro2::msleep( long duration)
+{
+    struct timespec ts;
+    int res;
+
+    if (duration < 0)
+    {
+        errno = EINVAL;
+        return -1;
+    }
+
+    ts.tv_sec = duration / 1000;
+    ts.tv_nsec = (duration % 1000) * 1000000;
+
+    do
+    {
+        res = nanosleep(&ts, &ts);
+    }
+    while (res && errno == EINTR);
+
+    return res;
+}
+
+void MyFocuserPro2::clearbufferonerror()
+{
+    bool rc;
+    char res[ML_RES] = {0};
+    int nbytes_read = 0;
+
+    // if the driver and controller get out of sync, this is an attempt to recover from that situation
+    // attempt to clear the receive buffer if the controller response arrived after a serial timeout
+    pthread_mutex_lock(&cmdlock);
+    msleep(MYFOCUSERPRO2_RECOVER_DELAY);
+    if (getActiveConnection()->type() == Connection::Interface::CONNECTION_SERIAL)
+    {
+        rc = tty_nread_section(PortFD, res, ML_RES, ML_DEL, MYFOCUSERPRO2_SERIAL_TIMEOUT, &nbytes_read);
+    }
+    else
+    {
+        // assume TCP/IP Connection
+        rc = tty_nread_section(PortFD, res, ML_RES, ML_DEL, MYFOCUSERPRO2_TCPIP_TIMEOUT, &nbytes_read);
+    }
+    pthread_mutex_unlock(&cmdlock);
+    if( rc != TTY_OK)
+    {
+        //LOGF_ERROR("No data read from controller");
+    }
+    else
+    {
+        LOGF_ERROR("Data read from controller: %s.", res);
+    }
+}
+
 bool MyFocuserPro2::sendCommand(const char * cmd, char * res)
 {
     int nbytes_written = 0, nbytes_read = 0, rc = -1;
@@ -1275,28 +1445,51 @@ bool MyFocuserPro2::sendCommand(const char * cmd, char * res)
 
     LOGF_DEBUG("CMD <%s>", cmd);
 
+    pthread_mutex_lock(&cmdlock);
     if ((rc = tty_write_string(PortFD, cmd, &nbytes_written)) != TTY_OK)
     {
         char errstr[MAXRBUF] = {0};
         tty_error_msg(rc, errstr, MAXRBUF);
-        LOGF_ERROR("Serial write error: %s.", errstr);
+        LOGF_ERROR("Connection write error: %s.", errstr);
+        pthread_mutex_unlock(&cmdlock);
         return false;
     }
 
     if (res == nullptr)
+    {
+        tcdrain(PortFD);
+        pthread_mutex_unlock(&cmdlock);
         return true;
+    }
+    //pthread_mutex_unlock(&cmdlock);
 
-    if ((rc = tty_nread_section(PortFD, res, ML_RES, ML_DEL, ML_TIMEOUT, &nbytes_read)) != TTY_OK)
+    // sleep for 20ms to give time for controller to process command before attempting to read reply
+    msleep(MYFOCUSERPRO2_SMALL_DELAY);
+
+    //pthread_mutex_lock(&cmdlock);
+    // replace ML_TIMEOUT, needs to be dependant on connection type
+    if (getActiveConnection()->type() == Connection::Interface::CONNECTION_SERIAL)
+    {
+        rc = tty_nread_section(PortFD, res, ML_RES, ML_DEL, MYFOCUSERPRO2_SERIAL_TIMEOUT, &nbytes_read);
+    }
+    else
+    {
+        // assume TCP/IP Connection
+        rc = tty_nread_section(PortFD, res, ML_RES, ML_DEL, MYFOCUSERPRO2_TCPIP_TIMEOUT, &nbytes_read);
+    }
+
+    if (rc != TTY_OK)
     {
         char errstr[MAXRBUF] = {0};
         tty_error_msg(rc, errstr, MAXRBUF);
-        LOGF_ERROR("Serial read error: %s.", errstr);
+        LOGF_ERROR("Connection read error: %s.", errstr);
+        pthread_mutex_unlock(&cmdlock);
         return false;
     }
 
     LOGF_DEBUG("RES <%s>", res);
 
     tcflush(PortFD, TCIOFLUSH);
-
+    pthread_mutex_unlock(&cmdlock);
     return true;
 }
