@@ -3,7 +3,7 @@
  Copyright(c) 2014 Jasem Mutlaq. All rights reserved.
 
  The code used calculate dome target AZ and ZD is written by Ferran Casarramona, and adapted from code from Markus Wildi.
- The transformations are based on the paper Matrix Method for Coodinates Transformation written by Toshimi Taki (http://www.asahi-net.or.jp/~zs3t-tk).
+ The transformations are based on the paper Matrix Method for Coordinates Transformation written by Toshimi Taki (http://www.asahi-net.or.jp/~zs3t-tk).
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
@@ -23,8 +23,8 @@
 #pragma once
 
 #include "defaultdevice.h"
-
-#include <libnova/ln_types.h>
+#include "libastro.h"
+#include "inditimer.h"
 
 #include <string>
 
@@ -63,7 +63,7 @@ class TCP;
 
    Developers need to subclass INDI::Dome to implement any driver for Domes within INDI.
 
-  \note The code used calculate dome target AZ and ZD is written by Ferran Casarramona, and adapted from code from Markus Wildi. The transformations are based on the paper Matrix Method for Coodinates
+  \note The code used calculate dome target AZ and ZD is written by Ferran Casarramona, and adapted from code from Markus Wildi. The transformations are based on the paper Matrix Method for Coordinates
  Transformation written by Toshimi Taki (http://www.asahi-net.or.jp/~zs3t-tk).
 
 \author Jasem Mutlaq
@@ -116,6 +116,13 @@ class Dome : public DefaultDevice
             SHUTTER_CLOSE /*!< Close Shutter */
         } ShutterOperation;
 
+        /*! Mount Locking Policy */
+        enum MountLockingPolicy
+        {
+            MOUNT_IGNORED,      /*!< Mount is ignored. Dome can park or unpark irrespective of mount parking status */
+            MOUNT_LOCKS,        /*!< Mount Locks. Dome can park if mount is completely parked first. */
+        };
+
         /** \typedef DomeState
                 \brief Dome status
             */
@@ -146,13 +153,14 @@ class Dome : public DefaultDevice
 
         enum
         {
-            DOME_CAN_ABORT    = 1 << 0, /*!< Can the dome motion be aborted? */
-            DOME_CAN_ABS_MOVE = 1 << 1, /*!< Can the dome move to an absolute azimuth position? */
-            DOME_CAN_REL_MOVE = 1 << 2, /*!< Can the dome move to a relative position a number of degrees away from current position? Positive degress is Clockwise direction. Negative Degrees is counter clock wise direction */
-            DOME_CAN_PARK     = 1 << 3, /*!< Can the dome park and unpark itself? */
-            DOME_CAN_SYNC     = 1 << 4, /*!< Can the dome sync to arbitrary postion? */
-            DOME_HAS_SHUTTER  = 1 << 5, /*!< Does the dome has a shutter than can be opened and closed electronically? */
-            DOME_HAS_VARIABLE_SPEED = 1 << 6 /*!< Can the dome move in different configurable speeds? */
+            DOME_CAN_ABORT          = 1 << 0, /*!< Can the dome motion be aborted? */
+            DOME_CAN_ABS_MOVE       = 1 << 1, /*!< Can the dome move to an absolute azimuth position? */
+            DOME_CAN_REL_MOVE       = 1 << 2, /*!< Can the dome move to a relative position a number of degrees away from current position? Positive degrees is Clockwise direction. Negative Degrees is counter clock wise direction */
+            DOME_CAN_PARK           = 1 << 3, /*!< Can the dome park and unpark itself? */
+            DOME_CAN_SYNC           = 1 << 4, /*!< Can the dome sync to arbitrary position? */
+            DOME_HAS_SHUTTER        = 1 << 5, /*!< Does the dome has a shutter than can be opened and closed electronically? */
+            DOME_HAS_VARIABLE_SPEED = 1 << 6, /*!< Can the dome move in different configurable speeds? */
+            DOME_HAS_BACKLASH       = 1 << 7  /*!< Can the dome compensate for backlash? */
         };
 
         /** \struct DomeConnection
@@ -168,16 +176,15 @@ class Dome : public DefaultDevice
         Dome();
         virtual ~Dome();
 
-        virtual bool initProperties();
-        virtual void ISGetProperties(const char * dev);
-        virtual bool updateProperties();
-        virtual bool ISNewNumber(const char * dev, const char * name, double values[], char * names[], int n);
-        virtual bool ISNewSwitch(const char * dev, const char * name, ISState * states, char * names[], int n);
-        virtual bool ISNewText(const char * dev, const char * name, char * texts[], char * names[], int n);
-        virtual bool ISSnoopDevice(XMLEle * root);
+        virtual bool initProperties() override;
+        virtual void ISGetProperties(const char * dev) override;
+        virtual bool updateProperties() override;
+        virtual bool ISNewNumber(const char * dev, const char * name, double values[], char * names[], int n) override;
+        virtual bool ISNewSwitch(const char * dev, const char * name, ISState * states, char * names[], int n) override;
+        virtual bool ISNewText(const char * dev, const char * name, char * texts[], char * names[], int n) override;
+        virtual bool ISSnoopDevice(XMLEle * root) override;
 
         static void buttonHelper(const char * button_n, ISState state, void * context);
-        static void updateMountCoordsHelper(void *context);
 
         /**
              * @brief setDomeConnection Set Dome connection mode. Child class should call this in the constructor before Dome registers
@@ -214,7 +221,7 @@ class Dome : public DefaultDevice
         }
 
         /**
-             * @return True if dome has absolute postion encoders.
+             * @return True if dome has absolute position encoders.
              */
         bool CanAbsMove()
         {
@@ -261,6 +268,14 @@ class Dome : public DefaultDevice
             return capability & DOME_HAS_VARIABLE_SPEED;
         }
 
+        /**
+         * @return True if the dome supports backlash
+         */
+        bool HasBacklash()
+        {
+            return capability & DOME_HAS_BACKLASH;
+        }
+
         /**
              * @brief isLocked, is the dome currently locked?
              * @return True if lock status equals true, and TelescopeClosedLockTP is Telescope Locks.
@@ -269,17 +284,16 @@ class Dome : public DefaultDevice
 
         DomeState getDomeState() const
         {
-            return domeState;
+            return m_DomeState;
         }
         void setDomeState(const DomeState &value);
 
         ShutterState getShutterState() const
         {
-            return shutterState;
+            return m_ShutterState;
         }
         void setShutterState(const ShutterState &value);
 
-        IPState getWeatherState() const;
         IPState getMountState() const;
 
     protected:
@@ -337,6 +351,20 @@ class Dome : public DefaultDevice
              */
         virtual IPState UnPark();
 
+        /**
+         * @brief SetBacklash Set the dome backlash compensation value
+         * @param steps value in absolute steps to compensate
+         * @return True if successful, false otherwise.
+         */
+        virtual bool SetBacklash(int32_t steps);
+
+        /**
+         * @brief SetBacklashEnabled Enables or disables the dome backlash compensation
+         * @param enable flag to enable or disable backlash compensation
+         * @return True if successful, false otherwise.
+         */
+        virtual bool SetBacklashEnabled(bool enabled);
+
         /**
              * \brief Open or Close shutter
              * \param operation Either open or close the shutter.
@@ -392,13 +420,13 @@ class Dome : public DefaultDevice
 
         /**
              * @brief SetRAPark Set current AZ parking position. The data park file (stored in ~/.indi/ParkData.xml) is updated in the process.
-             * @param value current Axis 1 value (AZ either in angles or encoder values as specificed by the DomeParkData type).
+             * @param value current Axis 1 value (AZ either in angles or encoder values as specified by the DomeParkData type).
              */
         void SetAxis1Park(double value);
 
         /**
              * @brief SetAxis1Park Set default AZ parking position.
-             * @param value Default Axis 1 value (AZ either in angles or encoder values as specificed by the DomeParkData type).
+             * @param value Default Axis 1 value (AZ either in angles or encoder values as specified by the DomeParkData type).
              */
         void SetAxis1ParkDefault(double steps);
 
@@ -477,7 +505,7 @@ class Dome : public DefaultDevice
              * @param fp pointer to configuration file
              * @return true if successful, false otherwise.
              */
-        virtual bool saveConfigItems(FILE * fp);
+        virtual bool saveConfigItems(FILE * fp) override;
 
         /**
              * @brief updateCoords updates the horizontal coordinates (Az & Alt) of the mount from the snooped RA, DEC and observer's location.
@@ -493,66 +521,84 @@ class Dome : public DefaultDevice
         /** \brief perform handshake with device to check communication */
         virtual bool Handshake();
 
+        /**
+         * Signal to concrete driver when Active Devices are updated.
+         */
+        virtual void ActiveDevicesUpdated() {};
+
         double Csc(double x);
         double Sec(double x);
 
-        INumberVectorProperty DomeSpeedNP;
-        INumber DomeSpeedN[1];
+        INDI::PropertyNumber DomeSpeedNP {1};
 
-        ISwitchVectorProperty DomeMotionSP;
-        ISwitch DomeMotionS[2];
+        INDI::PropertySwitch DomeMotionSP {2};
 
-        INumberVectorProperty DomeAbsPosNP;
-        INumber DomeAbsPosN[1];
+        INDI::PropertyNumber DomeAbsPosNP {1};
 
-        INumberVectorProperty DomeRelPosNP;
-        INumber DomeRelPosN[1];
+        INDI::PropertyNumber DomeRelPosNP {1};
 
-        ISwitchVectorProperty AbortSP;
-        ISwitch AbortS[1];
+        INDI::PropertySwitch AbortSP {1};
 
-        INumberVectorProperty DomeParamNP;
-        INumber DomeParamN[1];
+        INDI::PropertyNumber DomeParamNP {1};
 
-        INumberVectorProperty DomeSyncNP;
-        INumber DomeSyncN[1];
+        INDI::PropertyNumber DomeSyncNP {1};
 
-        ISwitchVectorProperty DomeShutterSP;
-        ISwitch DomeShutterS[2];
+        INDI::PropertySwitch DomeShutterSP {2};
 
-        ISwitchVectorProperty ParkSP;
-        ISwitch ParkS[2];
+        INDI::PropertySwitch ParkSP {2};
 
-        INumber ParkPositionN[1];
-        INumberVectorProperty ParkPositionNP;
+        INDI::PropertyNumber ParkPositionNP {1};
 
-        ISwitch ParkOptionS[3];
-        ISwitchVectorProperty ParkOptionSP;
+        INDI::PropertySwitch ParkOptionSP {3};
 
-        ISwitch AutoParkS[2];
-        ISwitchVectorProperty AutoParkSP;
+        INDI::PropertyText ActiveDeviceTP {3};
+        enum
+        {
+            ACTIVE_MOUNT,
+            ACTIVE_INPUT,
+            ACTIVE_OUTPUT
+        };
+
+        // Switch to lock id mount is unparked
+        INDI::PropertySwitch MountPolicySP {2};
+
+        // Shutter control on Park/Unpark
+        INDI::PropertySwitch ShutterParkPolicySP {2};
+        enum
+        {
+            SHUTTER_CLOSE_ON_PARK,
+            SHUTTER_OPEN_ON_UNPARK,
+        };
+
+        INDI::PropertyNumber PresetNP {3};
+        INDI::PropertySwitch PresetGotoSP {3};
+
+        INDI::PropertyNumber DomeMeasurementsNP {6};
+
+        INDI::PropertySwitch OTASideSP {5};
+        // 0 is East, 1 is West, 2 is as reported by mout, 3 as deducted by Hour Angle
+        // 4 ignore pier side and perform as in a fork mount
+        enum
+        {
+            DM_OTA_SIDE_EAST,
+            DM_OTA_SIDE_WEST,
+            DM_OTA_SIDE_MOUNT,
+            DM_OTA_SIDE_HA,
+            DM_OTA_SIDE_IGNORE
+        };
+
+        int mountOTASide = 0; // Side of the telescope with respect of the mount, 1: west, -1: east, 0 not reported
+        INDI::PropertySwitch DomeAutoSyncSP {2};
+
+        // Backlash toggle
+        INDI::PropertySwitch DomeBacklashSP {2};
+
+        // Backlash steps
+        INDI::PropertyNumber DomeBacklashNP {1};
 
         uint32_t capability;
         DomeParkData parkDataType;
 
-        ITextVectorProperty ActiveDeviceTP;
-        IText ActiveDeviceT[2] {};
-
-        // Switch to lock id mount is unparked
-        ISwitchVectorProperty TelescopeClosedLockTP;
-        ISwitch TelescopeClosedLockT[2];
-
-        INumber PresetN[3];
-        INumberVectorProperty PresetNP;
-        ISwitch PresetGotoS[3];
-        ISwitchVectorProperty PresetGotoSP;
-        INumber DomeMeasurementsN[6];
-        INumberVectorProperty DomeMeasurementsNP;
-        ISwitchVectorProperty OTASideSP;
-        ISwitch OTASideS[2];
-        ISwitchVectorProperty DomeAutoSyncSP;
-        ISwitch DomeAutoSyncS[2];
-
         double prev_az, prev_alt, prev_ra, prev_dec;
 
         // For Serial and TCP connections
@@ -562,19 +608,18 @@ class Dome : public DefaultDevice
         Connection::TCP * tcpConnection       = nullptr;
 
         // States
-        DomeState domeState;
-        ShutterState shutterState;
-        IPState mountState;
-        IPState weatherState;
+        DomeState m_DomeState;
+        ShutterState m_ShutterState;
+        IPState m_MountState;
 
         // Observer geographic coords. Snooped from mount driver.
-        struct ln_lnlat_posn observer;
+        IGeographicCoordinates observer;
         // Do we have valid geographic coords from mount driver?
         bool HaveLatLong = false;
 
         // Mount horizontal and equatorial coords. Snoops from mount driver.
-        struct ln_hrz_posn mountHoriztonalCoords;
-        struct ln_equ_posn mountEquatorialCoords;
+        INDI::IHorizontalCoordinates mountHoriztonalCoords;
+        INDI::IEquatorialCoordinates mountEquatorialCoords;
         // Do we have valid coords from mount driver?
         bool HaveRaDec = false;
 
@@ -602,13 +647,15 @@ class Dome : public DefaultDevice
         Controller * controller = nullptr;
 
         bool IsParked = false;
-        bool IsMountParked = false;
+        bool IsMountParked = true;
         bool IsLocked = true;
         bool AutoSyncWarning = false;
+        bool UseHourAngle = false;
 
         const char * ParkDeviceName;
         const std::string ParkDataFileName;
-        int m_HorizontalUpdateTimerID { -1 };
+        INDI::Timer m_MountUpdateTimer;
+        //int m_HorizontalUpdateTimerID { -1 };
         XMLEle * ParkdataXmlRoot, *ParkdeviceXml, *ParkstatusXml, *ParkpositionXml, *ParkpositionAxis1Xml;
 
         double Axis1ParkPosition;
