@@ -38,7 +38,7 @@ FilterInterface::~FilterInterface()
 void FilterInterface::initProperties(const char *groupName)
 {
     // @INDI_STANDARD_PROPERTY@
-    FilterSlotNP[0].fill("FILTER_SLOT_VALUE", "Filter", "%3.0f", 1.0, 12.0, 1.0, 1.0);
+    FilterSlotNP[0].fill("FILTER_SLOT_VALUE", "Filter", "%.f", 1.0, 12, 1.0, 1.0);
     FilterSlotNP.fill(m_defaultDevice->getDeviceName(), "FILTER_SLOT", "Filter Slot", groupName, IP_RW, 60, IPS_IDLE);
 
     loadFilterNames();
@@ -89,8 +89,9 @@ bool FilterInterface::processNumber(const char *dev, const char *name, double va
         if (TargetFilter < FilterSlotNP[0].getMin() || TargetFilter > FilterSlotNP[0].getMax())
         {
             FilterSlotNP.setState(IPS_ALERT);
-            DEBUGFDEVICE(m_defaultDevice->getDeviceName(), Logger::DBG_ERROR, "Error: valid range of filter is from %g to %g",
-                         FilterSlotNP[0].getMin(), FilterSlotNP[0].getMax());
+            DEBUGFDEVICE(m_defaultDevice->getDeviceName(), Logger::DBG_ERROR,
+                         "Error: requested slot %d is invalid (Range from %.f to %.f",
+                         static_cast<int>(TargetFilter), FilterSlotNP[0].getMin(), FilterSlotNP[0].getMax());
             FilterSlotNP.apply();
             return false;
         }
@@ -114,35 +115,6 @@ bool FilterInterface::processText(const char *dev, const char *name, char *texts
 {
     if (dev && !strcmp(dev, m_defaultDevice->getDeviceName()) && !strcmp(name, "FILTER_NAME"))
     {
-        // If this call due to config loading, let's delete existing dummy property and define the full one
-        if (loadingFromConfig)
-        {
-            loadingFromConfig = false;
-            m_defaultDevice->deleteProperty("FILTER_NAME");
-
-            char filterName[MAXINDINAME];
-            char filterLabel[MAXINDILABEL];
-
-            FilterNameTP.resize(0);
-
-            for (int i = 0; i < n; i++)
-            {
-                snprintf(filterName, MAXINDINAME, "FILTER_SLOT_NAME_%d", i + 1);
-                snprintf(filterLabel, MAXINDILABEL, "Filter#%d", i + 1);
-
-                INDI::WidgetText oneText;
-                oneText.fill(filterName, filterLabel, texts[i]);
-                FilterNameTP.push(std::move(oneText));
-            }
-
-            // @INDI_STANDARD_PROPERTY@
-            FilterNameTP.fill(m_defaultDevice->getDeviceName(), "FILTER_NAME", "Filter",
-                              FilterSlotNP.getGroupName(), IP_RW, 0, IPS_IDLE);
-            FilterNameTP.shrink_to_fit();
-            m_defaultDevice->defineProperty(FilterNameTP);
-            return true;
-        }
-
         FilterNameTP.update(texts, names, n);
         FilterNameTP.setState(IPS_OK);
 
@@ -174,12 +146,9 @@ bool FilterInterface::saveConfigItems(FILE *fp)
 
 void FilterInterface::SelectFilterDone(int f)
 {
-    //  The hardware has finished changing
-    //  filters
+    //  The hardware has finished changing  filters
     FilterSlotNP[0].setValue(f);
     FilterSlotNP.setState(IPS_OK);
-    // Tell the clients we are done, and
-    //  filter is now useable
     FilterSlotNP.apply();
 }
 
@@ -210,27 +179,16 @@ void FilterInterface::generateSampleFilters()
 
 bool FilterInterface::GetFilterNames()
 {
-    // Load from config
-    if (FilterNameTP.size() == 0)
-    {
+    // Generate sample filters if nothing available.
+    if (FilterNameTP.isEmpty())
         generateSampleFilters();
 
-        //        // JM 2018-07-09: Set loadingFromConfig to true here before calling loadConfig
-        //        // since if loadConfig is successful, ISNewText could be executed _before_ we have a chance
-        //        // to set loadFromConfig below
-        //        loadingFromConfig = true;
-
-        //        // If property is found, let's define it once loaded to the client and delete
-        //        // the generate sample filters above
-        //        loadingFromConfig = m_defaultDevice->loadConfig(true, "FILTER_NAME");
-    }
-
     return true;
 }
 
 bool FilterInterface::SetFilterNames()
 {
-    return m_defaultDevice->saveConfig(true, "FILTER_NAME");
+    return m_defaultDevice->saveConfig(FilterNameTP);
 }
 
 bool FilterInterface::loadFilterNames()
@@ -302,6 +260,10 @@ bool FilterInterface::loadFilterNames()
                       FilterSlotNP.getGroupName(), IP_RW, 0, IPS_IDLE);
     FilterNameTP.shrink_to_fit();
 
+    // Filter Slot must match and agree with filter name if it was loaded successfully.
+    if (FilterNameTP.count() > 0)
+        FilterSlotNP[0].setMax(FilterNameTP.count());
+
     fclose(fp);
     delXMLEle(fproot);
     delLilXML(lp);
