@@ -24,10 +24,6 @@
 #include "stream/streammanager.h"
 #include <fitsio.h>
 
-#ifdef HAVE_WEBSOCKET
-#include "indiwsserver.h"
-#endif
-
 #include <fitsio.h>
 
 #include <memory>
@@ -37,6 +33,8 @@
 #include <mutex>
 #include <thread>
 #include <stream/streammanager.h>
+#include <connectionplugins/connectionserial.h>
+#include <connectionplugins/connectiontcp.h>
 
 //JM 2019-01-17: Disabled until further notice
 //#define WITH_EXPOSURE_LOOPING
@@ -231,7 +229,7 @@ class SensorInterface : public DefaultDevice
         void setNAxis(int value);
 
         /**
-         * @brief setIntegrationExtension Set integration exntension
+         * @brief setIntegrationExtension Set integration extension
          * @param ext extension (fits, jpeg, raw..etc)
          */
         void setIntegrationFileExtension(const char *ext);
@@ -291,7 +289,7 @@ class SensorInterface : public DefaultDevice
         virtual bool StartIntegration(double duration);
 
         /**
-         * \brief Uploads target Device exposed buffer as FITS to the client. Dervied classes should class
+         * \brief Uploads target Device exposed buffer as FITS to the client. Derived classes should class
          * this function when an Integration is complete.
          * @param targetDevice device that contains upload integration data
          * \note This function is not implemented in Sensor, it must be implemented in the child class
@@ -320,7 +318,7 @@ class SensorInterface : public DefaultDevice
          * \brief Add FITS keywords to a fits file
          * \param fptr pointer to a valid FITS file.
          * \param buf The buffer of the fits contents.
-         * \param len The lenght of the buffer.
+         * \param len The length of the buffer.
          * \note In additional to the standard FITS keywords, this function write the following
          * keywords the FITS file:
          * <ul>
@@ -339,6 +337,25 @@ class SensorInterface : public DefaultDevice
         /** A function to just remove GCC warnings about deprecated conversion */
         void fits_update_key_s(fitsfile *fptr, int type, std::string name, void *p, std::string explanation, int *status);
 
+        /** Return the connection file descriptor */
+        int getPortFD()
+        {
+            return PortFD;
+        }
+
+        /** Export the serial connection object */
+        Connection::Serial *getSerialConnection()
+        {
+            return serialConnection;
+        }
+
+        /** Export the TCP connection object */
+        Connection::TCP *getTcpConnection()
+        {
+            return tcpConnection;
+        }
+
+
     protected:
 
         /**
@@ -407,7 +424,7 @@ class SensorInterface : public DefaultDevice
         }
 
         /**
-         * @brief SetCapability Set the Sensor capabilities. Al fields must be initilized.
+         * @brief SetCapability Set the Sensor capabilities. Al fields must be initialized.
          * @param cap pointer to SensorCapability struct.
          */
         void SetCapability(uint32_t cap);
@@ -511,9 +528,6 @@ class SensorInterface : public DefaultDevice
 
         std::unique_ptr<StreamManager> Streamer;
         std::unique_ptr<DSP::Manager> DSP;
-
-
-
         Connection::Serial *serialConnection = NULL;
         Connection::TCP *tcpConnection       = NULL;
 
