@@ -340,6 +340,15 @@ int IUFindOnSwitchIndex(const ISwitchVectorProperty *svp)
     return -1;
 }
 
+/* Find index of the ON member of ISState array */
+int IUFindOnStateIndex(ISState *states, int n)
+{
+    for (int i = 0; i < n; i++)
+        if (states[i] == ISS_ON)
+            return i;
+    return -1;
+}
+
 /* Find name the ON member in the given states and names */
 const char *IUFindOnSwitchName(ISState *states, char *names[], int n)
 {
