@@ -22,20 +22,20 @@ class MockCCDSimDriver: public CCDSim
 
         void testProperties()
         {
-            INumberVectorProperty * const p = getNumber("SIMULATOR_SETTINGS");
+            auto p = getNumber("SIMULATOR_SETTINGS");
             ASSERT_NE(p, nullptr);
-            ASSERT_NE(IUFindNumber(p, "SIM_XRES"), nullptr);
-            ASSERT_NE(IUFindNumber(p, "SIM_YRES"), nullptr);
-            ASSERT_NE(IUFindNumber(p, "SIM_XSIZE"), nullptr);
-            ASSERT_NE(IUFindNumber(p, "SIM_YSIZE"), nullptr);
-            ASSERT_NE(IUFindNumber(p, "SIM_MAXVAL"), nullptr);
-            ASSERT_NE(IUFindNumber(p, "SIM_SATURATION"), nullptr);
-            ASSERT_NE(IUFindNumber(p, "SIM_LIMITINGMAG"), nullptr);
-            ASSERT_NE(IUFindNumber(p, "SIM_NOISE"), nullptr);
-            ASSERT_NE(IUFindNumber(p, "SIM_SKYGLOW"), nullptr);
-            ASSERT_NE(IUFindNumber(p, "SIM_OAGOFFSET"), nullptr);
-            ASSERT_NE(IUFindNumber(p, "SIM_POLAR"), nullptr);
-            ASSERT_NE(IUFindNumber(p, "SIM_POLARDRIFT"), nullptr);
+            ASSERT_NE(p.findWidgetByName("SIM_XRES"), nullptr);
+            ASSERT_NE(p.findWidgetByName("SIM_YRES"), nullptr);
+            ASSERT_NE(p.findWidgetByName("SIM_XSIZE"), nullptr);
+            ASSERT_NE(p.findWidgetByName("SIM_YSIZE"), nullptr);
+            ASSERT_NE(p.findWidgetByName("SIM_MAXVAL"), nullptr);
+            ASSERT_NE(p.findWidgetByName("SIM_SATURATION"), nullptr);
+            ASSERT_NE(p.findWidgetByName("SIM_LIMITINGMAG"), nullptr);
+            ASSERT_NE(p.findWidgetByName("SIM_NOISE"), nullptr);
+            ASSERT_NE(p.findWidgetByName("SIM_SKYGLOW"), nullptr);
+            ASSERT_NE(p.findWidgetByName("SIM_OAGOFFSET"), nullptr);
+            ASSERT_NE(p.findWidgetByName("SIM_POLAR"), nullptr);
+            ASSERT_NE(p.findWidgetByName("SIM_POLARDRIFT"), nullptr);
         }
 
         void testGuideAPI()
@@ -88,22 +88,22 @@ class MockCCDSimDriver: public CCDSim
             int const maxval = pow(2, 8) - 1;
 
             // Setup a 65x65, 16-bit depth, 4.6u square pixel sensor
-            INumberVectorProperty * const p = getNumber("SIMULATOR_SETTINGS");
+            auto p = getNumber("SIMULATOR_SETTINGS");
             ASSERT_NE(p, nullptr);
-            IUFindNumber(p, "SIM_XRES")->value = (double) xres;
-            IUFindNumber(p, "SIM_YRES")->value = (double) yres;
+            p.findWidgetByName("SIM_XRES")->setValue((double) xres);
+            p.findWidgetByName("SIM_YRES")->setValue((double) yres);
             // There is no way to set depth, it is hardcoded at 16-bit - so set maximum value instead
-            IUFindNumber(p, "SIM_MAXVAL")->value = (double) maxval;
-            IUFindNumber(p, "SIM_XSIZE")->value = 4.6;
-            IUFindNumber(p, "SIM_YSIZE")->value = 4.6;
+            p.findWidgetByName("SIM_MAXVAL")->setValue((double) maxval);
+            p.findWidgetByName("SIM_XSIZE")->setValue(4.6);
+            p.findWidgetByName("SIM_YSIZE")->setValue(4.6);
 
             // Setup a saturation magnitude (max ADUs in one second) and limit magnitude (zero ADU whatever the exposure)
-            IUFindNumber(p, "SIM_SATURATION")->value = 0.0;
-            IUFindNumber(p, "SIM_LIMITINGMAG")->value = 30.0;
+            p.findWidgetByName("SIM_SATURATION")->setValue(0.0);
+            p.findWidgetByName("SIM_LIMITINGMAG")->setValue(30.0);
 
             // Setup some parameters to simplify verifications
-            IUFindNumber(p, "SIM_SKYGLOW")->value = 0.0;
-            IUFindNumber(p, "SIM_NOISE")->value = 0.0;
+            p.findWidgetByName("SIM_SKYGLOW")->setValue(0.0);
+            p.findWidgetByName("SIM_NOISE")->setValue(0.0);
             this->seeing = 1.0f; // No way to control seeing from properties
 
             // Setup
